/***********************************************************************
 * REVISION LIST:
 * 
 * REV 06 - Converted script for use with Historian v10.0 database structures.
 *        - Script will abort if Database Version is older than v10.0
 *
 * REV 05a - Added code to abort if Database Version is greater than 9.x
 * REV 05b - Added code to include all 'SysDataAcq%' tags (previously only did 'SysDataAcq0%')
 *
 * REV 04 - Added scripts to manually rebuild the tag hierarchy without having to redeploy an engine.
 *
 * REV 03 - Added scripts to remove duplicates in aaObjectData and aaAttributeData tables
 *
 * REV 02 - Added scripts to backup and restore custom-created, 
 *          non-AppServer namespaces.
 * 
 * REV 01 - Added "USE Runtime" and cleaned up inconsistent formatting
 * 
 ***********************************************************************/


/* REV 01 */
USE Runtime
GO

/* REV 06 */
--NOTE: StringValue of '10,0' is less than '7,0' even though it is a newer version...
IF EXISTS (SELECT Value FROM dbo.SystemParameter WHERE Name = 'DatabaseVersion' AND Value > '7,0')
  RAISERROR('Sorry, this script should only be used for InSQL/Historian v10.0; your connection will now be terminated.', 20, -1) WITH LOG
GO


/* Rev 03 */
DELETE from aaAttributeData
WHERE wwDomainTagKey NOT IN (select wwDomainTagKey from TagRef)
GO

DELETE from aaOD1
FROM aaObjectData aaOD1, aaObjectData aaOD2
WHERE aaOD1.aaTagname = aaOD2.aaTagname
AND aaOD1.ObjectKey > aaOD2.ObjectKey
GO

/* REV 02a - backup custom namespaces & tags */
/* REV 06 - backup replication namespaces & tags too */

if exists(
    select 1 from sys.sysobjects
    where id = object_id(N'dbo.BKP_PublicNameSpace')
        and OBJECTPROPERTY(id, N'IsUserTable') = 1)
begin
    drop table dbo.BKP_PublicNameSpace;
end;
go

if exists(
    select 1 from dbo.sysobjects
    where id = object_id(N'dbo.BKP_PublicGroupTag')
        and OBJECTPROPERTY(id, N'IsUserTable') = 1)
begin
    drop table dbo.BKP_PublicGroupTag;
end;
go

if exists(
    select 1 from sys.sysobjects
    where id = object_id(N'dbo.BKP_PublicNameSpaceSystem')
        and OBJECTPROPERTY(id, N'IsUserTable') = 1)
begin
    drop table dbo.BKP_PublicNameSpaceSystem;
end;
go

SELECT * 
INTO BKP_PublicNameSpaceSystem
FROM PublicNameSpace
WHERE Type = 1000000
    and Name in (
        N'Replication',
        N'All State Summary Tags',
        N'Replication Sources');
go

SELECT * 
INTO BKP_PublicNameSpace
FROM PublicNameSpace
WHERE Type >= 2000000

SELECT * 
INTO BKP_PublicGroupTag
FROM PublicGroupTag
WHERE NameKey IN (SELECT NameKey FROM BKP_PublicNameSpace)

--<remove leading dashes for testing>--
-- SELECT * FROM BKP_PublicNameSpace
-- SELECT * FROM BKP_PublicNameSpaceSystem
-- SELECT * FROM BKP_PublicGroupTag
GO

/* REV 06 - Drop Constraints so tables can be truncated  */

if exists(
    select 1 from sys.foreign_keys
    where object_id = OBJECT_ID(N'dbo.FK_PublicGroupTag_PublicNameSpace')
        and parent_object_id = OBJECT_ID(N'dbo.PublicGroupTag'))
begin
    alter table dbo.PublicGroupTag drop constraint FK_PublicGroupTag_PublicNameSpace;
end;

if exists(
    select 1 from sys.foreign_keys
    where object_id = OBJECT_ID(N'dbo.FK_PublicGroupTag_TagRef')
        and parent_object_id = OBJECT_ID(N'dbo.PublicGroupTag'))
begin
    alter table dbo.PublicGroupTag drop constraint FK_PublicGroupTag_TagRef;
end;
go

/* REV 0 */

truncate table dbo.PublicGroupTag
GO
truncate table dbo.PublicNameSpace
GO


/*** NOTE: Almost everything from this point forward comes from InSQL installation scripts.  DO NOT MODIFY!!! ***/
/*** vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv ***/

/*******************************************************************/
/* Public Namespace                                                */
/*******************************************************************/

-- IDENTITY 1 -------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000004,
    0,
    N'Public Groups',
    N'Public Groups');


/*******************************************************************/
/* System Variable Groups                                          */
/*******************************************************************/

-- IDENTITY 2 -------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    1,
    N'All Analog Tags',
    N'All Analog Tags');

-- IDENTITY 3 -------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    1,
    N'All Discrete Tags',
    N'All Discrete Tags');

-- IDENTITY 4 -------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    1,
    N'All String Tags',
    N'All String Tags');

-- IDENTITY 5 -------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    1,
    N'All Analog Summary Tags',
    N'All Analog Summary Tags');

-- IDENTITY 6 -------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    1,
    N'All Event Tags',
    N'All Event Tags');

-- IDENTITY 7 -------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    1,
    N'InTouch Nodes',
    N'InTouch Nodes');

-- IDENTITY 8 -------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    1,
    N'System Status Tags',
    N'System Status Tags');

-- IDENTITY 9 -------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    8,
    N'Error',
    N'Error')

-- IDENTITY 10 ------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    8,
    N'Date',
    N'Date');

-- IDENTITY 11 ------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    8,
    N'Time',
    N'Time');

-- IDENTITY 12 ------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    8,
    N'Space',
    N'Space');

-- IDENTITY 13 ------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    8,
    N'I/O Stats',
    N'I/O Stats');

-- IDENTITY 14 ------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    8,
    N'Event System',
    N'Event System');

-- IDENTITY 15 ------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    8,
    N'System Monitoring',
    N'System Monitoring');

-- IDENTITY 16 ------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    8,
    N'Performance Monitoring',
    N'Performance Monitoring');

-- IDENTITY 17 ------------------------------------------------------
insert into dbo.PublicNameSpace(
    Type,
    ParentKey,
    Name,
    OriginalName)
values(
    1000000,
    8,
    N'Others',
    N'Others');
go

SET IDENTITY_INSERT dbo.PublicNameSpace ON;

/* REV 06 */
-- This will insert 'Replication', 'All State Summary Tags' and 'Replication Sources' namespaces
insert into dbo.PublicNameSpace(
    NameKey,
    Type,
    ParentKey,
    Name,
    OriginalName)
select
    NameKey,
    Type,
    ParentKey,
    Name,
    OriginalName
from dbo.BKP_PublicNameSpaceSystem;

/* REV 02b - restore custom namespaces */
-- This will take care of all the custom namespaces that user has created and
-- also the Replication Sources
insert into dbo.PublicNameSpace(
    NameKey,
    Type,
    ParentKey,
    Name,
    OriginalName)
select
    NameKey,
    Type,
    ParentKey,
    Name,
    OriginalName
from dbo.BKP_PublicNameSpace;

/*******************************************************************/
SET IDENTITY_INSERT PublicNameSpace OFF
GO
/*******************************************************************/

/***********************************************************/
/* System Variables in Dedicated PublicGroupTag            */
/***********************************************************/

-- Other Variables ---------------------------------------------------
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    17,
    wwDomainTagKey
from dbo.TagRef
where TagName = N'SysMinutesRun';

insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    17,
    wwDomainTagKey 
from dbo.TagRef
where TagName = N'SysString';

insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    17,
    wwDomainTagKey 
from dbo.TagRef
where TagName = N'SysConfigStatus';

insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    17,
    wwDomainTagKey 
from dbo.TagRef
where TagName like 'SysHead%';

-- Error Variables --------------------------------------------------
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    9,
    wwDomainTagKey
from dbo.TagRef
where TagName like N'Sys%ErrCnt';

-- Date Variables ---------------------------------------------------
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    10,
    wwDomainTagKey
from dbo.TagRef
where TagName like N'SysDate%';

-- Time Variables ---------------------------------------------------
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    11,
    wwDomainTagKey
from dbo.TagRef
where TagName like N'SysTime%';

-- Space Variables --------------------------------------------------
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    12,
    wwDomainTagKey 
from dbo.TagRef
where TagName like N'SysSpace%';

-- I/O Stats Variables -----------------------------------------------
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    13,
    wwDomainTagKey
from dbo.TagRef
where TagName like N'SysStatus%Rx%';
go

-- Event Variables --------------------------------------------------
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    14,
    wwDomainTagKey
from dbo.TagRef
where TagName like N'SysEvent%';

insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    14,
    wwDomainTagKey
from dbo.TagRef
where TagName = N'SysStatusEvent';

-- System Monitoring --------------------------------------------------
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    15,
    wwDomainTagKey
from dbo.TagRef
where TagName in (
    N'SysEventSystem',
    N'SysRetrieval',
    N'SysInSQLIOS',
    N'SysPulse',
    N'SysConfiguration',
    N'SysStorage',
    N'SysManualStorage',
    N'SysSystemDriver',
    N'SysOLEDB',
    N'SysIndexing',
    N'SysReplication',
    N'SysTier2Storage',
    N'SysMDASServer');
go

-- Performance Monitoring --------------------------------------------------
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    16,
    wwDomainTagKey
from dbo.TagRef
where TagName like N'SysPerf%';
go

-- System Tags --------------------------------------------------

insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    17,
    wwDomainTagKey
from dbo.TagRef
where TagName in (
    N'SysHistoryCacheUsed',
    N'SysHistoryCacheFaults',
    N'SysHistoryClients',
    N'SysRateDeadbandForcedValues');

/* REV 05 */
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    13,
    wwDomainTagKey
from dbo.TagRef
--  WHERE  TagName IN ('SysDataAcq0BadValues', 'SysDataAcq0OutsideRealtime', 'SysDataAcqOverallItemsPerSec')
where TagName like N'SysDataAcq%'
	and TagType = 1;

insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    15,
    wwDomainTagKey
from dbo.TagRef
where TagName like N'SysDataAcq_'
	and TagType = 2;
go

/* REV 06 */
-- Replication tags
declare @nameKeyVal int;
select
    @nameKeyVal = NameKey
from dbo.PublicNameSpace
where Type = 1000000
    and Name = N'Replication';

insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    @nameKeyVal,
    wwDomainTagKey
from dbo.TagRef
where TagName like N'SysReplication_%';
go

--This will fix the Public namespace groups for "All Analog Tags", "All Discrete Tags", etc.
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    TagType + 1,
    wwDomainTagKey
from dbo.TagRef
where TagType <> 7;
go

/* REV 06 */
-- All Analog Summary Tags
-- All State Summary Tags
declare @allAnalogSummaryTagsPNSKey int;
declare @allStateSummaryTagsPNSKey int;

select
    @allAnalogSummaryTagsPNSKey = NameKey
from dbo.PublicNameSpace
where Name = N'All Analog Summary Tags'
    and ParentKey = 1;

select
    @allStateSummaryTagsPNSKey = NameKey
from dbo.PublicNameSpace
where Name = N'All State Summary Tags'
    and ParentKey = 1;

insert dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    NameKey = 
        case (stype.StructureTypeName)
            when N'AnalogSummaryInt16' then @allAnalogSummaryTagsPNSKey
            when N'AnalogSummaryUInt16' then @allAnalogSummaryTagsPNSKey
            when N'AnalogSummaryInt32' then @allAnalogSummaryTagsPNSKey
            when N'AnalogSummaryUInt32' then @allAnalogSummaryTagsPNSKey
            when N'AnalogSummaryFloat' then @allAnalogSummaryTagsPNSKey
            when N'AnalogSummaryDouble' then @allAnalogSummaryTagsPNSKey
            when N'StateSummary' then @allStateSummaryTagsPNSKey
            else 0 -- Error
        end,
    tr.wwDomainTagKey
from dbo.StructureTag i
inner join dbo.TagRef tr
    on i.TagName = tr.TagName
left join dbo.StructureType stype
    on i.StructureId = stype.StructureId;
go

/*** ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ ***/
/*** NOTE: Almost everything prior to this point comes from InSQL installation scripts.  DO NOT MODIFY!!! ***/


/* REV 02b - restore custom tags */
insert into dbo.PublicGroupTag(
    NameKey,
    wwDomainTagKey)
select
    NameKey,
    wwDomainTagKey
from dbo.BKP_PublicGroupTag
go

/* REV 04 - Rebuild tag hierarchy */
exec dbo.aaInternalNSAreaUpdate
go

exec dbo.aaInternalNSObjectUpdate
go

/* REV 06 */
-- Add the table Constraints that we deleted earlier
alter table dbo.PublicGroupTag
add constraint FK_PublicGroupTag_PublicNameSpace
foreign key(
    NameKey)
references dbo.PublicNameSpace(
    NameKey)
on delete cascade;
go

alter table dbo.PublicGroupTag
add constraint FK_PublicGroupTag_TagRef
foreign key(
    wwDomainTagKey)
references dbo.TagRef(
    wwDomainTagKey)
on delete cascade;
go

/*END*/